/***********************************************************

	LCD-Routinen
	fur Samsung 2138a mit M50530-Controller
	24 x 8 Zeichen

	Copyright 2003 by Daniel Jelkmann


	Zur Ansteuerung wird der 4Bit-Mode verwendet.
	Neben den 4 Datenleitungen (Data0-3) werden 4 weitere
	Steuerleitungen (Read/Write, Execute, OC1, OC2)
	benotigt. Damit werden am AVR 8 Pins benotigt.

	
	Anschluss:
	
	Alle 8 LCD-Signale mussen an einem Port des AVR
	angeschlossen werden. Gema? folgender Tabelle mussen
	die 4 Datenleitungen an den niederwertigsten Bits
	anliegen. Die Reihenfolge der restlichen 4 Steuerleitungen
	und der verwendete Port ist uber die Konstanten im 
	Quellcode festgelegt und muss ggf. angepasst werden.
	Die restlichen LCD-Signale sind in folgender Tabelle
	zu finden.

		LCD	Beschreibung		Anschluss
		-------------------------------------------------------------------------
		Pin 01	Masse			Masse
		Pin 02	Data0			nicht angeschlossen
		Pin 03	Data1			nicht angeschlossen
		Pin 04	Data2			nicht angeschlossen
		Pin 05	Data3			nicht angeschlossen
		Pin 06	Data4			AVR Port Bit 0
		Pin 07	Data5			AVR Port Bit 1
		Pin 08	Data6			AVR Port Bit 2
		Pin 09	Data7			AVR Port Bit 3
		Pin 10	EX (Execute)		AVR Port Bit 5*
		Pin 11	R/W (Read/Write)	AVR Port Bit 4*
		Pin 12	OC2				AVR Port Bit 7*
		Pin 13	OC1				AVR Port Bit 6*
		Pin 14	Kontrast-Regelung, ungefahr 8,2 Volt, am besten mittels Poti anschlie?en
		Pin 15	Versorgungsspannung, +5 Volt
		Pin 16	Masse			Masse


	
	* Die Reihenfolge dieser Leitungen kann uber die
	Konstanten im Quellcode angepasst werden.
	
	
	Es wird keine Gewahr fur die Vollstandigkeit, Korrektheit,
	Funktionsfahigkeit oder fur sonstige Eigenschaften des
	Codes ubernommen. Haftung ausgeschlossen.
	
*************************************************************/

#ifndef LCD_M50530_H
#define LCD_M50530_H

#define F_CPU 8000000UL

#include "stm32f10x.h"

#include "math.h"
#include "GPIO_macro.h"


// ein paar Konstanten zur besseren Lesbarkeit des Quelltextes :-)
#define LCD_DISPLAY_ON		16
#define LCD_CURSOR_ON		8
#define LCD_UNDERLINE		4
#define LCD_CURSOR_BLINK	2
#define LCD_CHARACTER_BLINK	1


#define LCD_CURSOR_ADDRESS_NO_CHANGE	64
#define LCD_CURSOR_ADDRESS_INC_RD		72
#define LCD_CURSOR_ADDRESS_INC_WD		80
#define LCD_CURSOR_ADDRESS_INC_RDWD		88
#define LCD_CURSOR_ADDRESS_DEC_RD		104
#define LCD_CURSOR_ADDRESS_DEC_WD		112
#define LCD_CURSOR_ADDRESS_DEC_RDWD		120

#define LCD_DISPLAY_START_ADDRESS_NO_CHANGE	64
#define LCD_DISPLAY_START_ADDRESS_INC_RD	65
#define LCD_DISPLAY_START_ADDRESS_INC_WD	66
#define LCD_DISPLAY_START_ADDRESS_INC_RDWD	67
#define LCD_DISPLAY_START_ADDRESS_DEC_RD	69
#define LCD_DISPLAY_START_ADDRESS_DEC_WD	70
#define LCD_DISPLAY_START_ADDRESS_DEC_RDWD	71

// LCD commands


#define LCDM50530_WC_Line1		(1<<6)
#define LCDM50530_WC_Line2		(2<<6)
#define LCDM50530_WC_Line3		(3<<6)
#define LCDM50530_WC_Line4   	(4<<6)

#define LCDM50530_SF_IO8n4		(1<<5)
#define LCDM50530_SF_FNT8n12	(1<<4)
#define LCDM50530_SF_DT0			(1<<2)
#define LCDM50530_SF_DT1			(1<<3)
#define LCDM50530_SF_RA0			(1<<0)
#define LCDM50530_SF_RA1			(1<<1)

#define LCDM50530_SE_CurDnnUp	(1<<5)
#define LCDM50530_SE_CurUDatW	(1<<4)
#define LCDM50530_SE_CurUDatR	(1<<3)
#define LCDM50530_SE_DSADnnUp	(1<<2)
#define LCDM50530_SE_DSAUDatW	(1<<1)
#define LCDM50530_SE_DSAUDatR	(1<<0)

#define LCDM50530_SD_LCDON		(1<<4)
#define LCDM50530_SD_CurON		(1<<3)
#define LCDM50530_SD_ULineON	(1<<2)
#define LCDM50530_SD_CurBLNK	(1<<1)
#define LCDM50530_SD_ChrBLNK	(1<<0)

#define LCDM50530_MA_ShifCur	(1<<3)
#define LCDM50530_MA_CurLeft	(1<<2)
#define LCDM50530_MA_ShifDSA	(1<<1)
#define LCDM50530_MA_DSALeft	(1<<0)

#define LCDM50530_SU_1	(1<<1)			////
#define LCDM50530_SU_0	(1<<0)			////

#define LCDM50530_WU_1	(1<<1)			////
#define LCDM50530_WU_0	(1<<0)			////

#define LCDM50530_SB_B1				(1<<1)
#define LCDM50530_SB_B0				(1<<0)


#define LCD_SPECIAL_AE						(char)144	// A, dezimal 144, oktal 220
#define LCD_SPECIAL_OE						(char)145	// O, dezimal 145, oktal 221
#define LCD_SPECIAL_UE						(char)197	// U, dezimal 197, oktal 305
#define LCD_SPECIAL_ae						(char)160	// a, dezimal 160, oktal 240
#define LCD_SPECIAL_oe						(char)161	// o, dezimal 161, oktal 241
#define LCD_SPECIAL_ue						(char)17	// u, dezimal 17, oktal 21
#define LCD_SPECIAL_ss						(char)163	// ?,  dezimal 163, oktal 243


// wartet einen kurzen Moment, 4 cycles per loop * 8 = 32 cycles, macht bei 16 MHz 2 s
#define delay_short()	Delay_ms(2)


// sendet den angelegten Befehl zum LCD
// setzt kurzzeitig das EX-Signal und nimmt es anschlie?end wieder zuruck
extern void LCD_execute(void);


// wartet solange bis das Busy-Flag nicht mehr gesetzt ist
// und das LCD weitere Befehle entgegennimmt
extern uint8_t LCD_waitReady(void);


// ubertragt das ubergebene Byte an das LCD
extern void LCD_sendByte(const unsigned char byte);

extern void LCD_sendUpperHalfByte(const unsigned char ddata);


// setzt 4-Bit-Mode und initialisiert das LCD
// muss als erstes aufgerufen werden, bevor das LCD angesteuert werden kann
extern void LCD_init(void);

// loscht die Anzeige und setzt die Display + Cursor-Adresse auf 0,0
extern void LCD_clear(void);


// setzt den Entry-Mode des LCDs, also die automatische Erhohung der Adresse nach einem Lese- oder Schreibbefehl
// man verwende die oben deklarierten Konstanten ;)
#define LCD_setEntryMode(mode)		do{M50530_CS2_Off;M50530_CS1_Off;M50530_RnW_Off;LCD_sendByte(mode);}while(0)


// schreibt das ubergebene Zeichen an die aktuelle Cursor-Position
#define LCD_writeChar(c)				do{M50530_CS2_On;M50530_CS1_Off;M50530_RnW_Off;LCD_sendByte(c);}while(0)


// schreibt die ubergebene Zeichenkette an die aktuelle Cursor-Position
extern void LCD_write(const char * c);

// schreibt die ubergebene Zeichenkette an die aktuelle Cursor-Position (fur String im ROM)
extern void LCD_write_P(const unsigned char * progmem_string);


// setzt den Display-Modus
// man verwende die oben deklarierten Konstanten ;)
#define LCD_setDisplay(mode)		do{M50530_CS1_Off;M50530_CS2_Off;M50530_RnW_Off;LCD_sendByte(mode | 32);}while(0)	


// setzt den Cursor an die angegebene Adresse
#define LCD_setCursorAddress(address)	do{M50530_CS1_On;M50530_CS2_On;M50530_RnW_Off;LCD_sendByte(address);}while(0)


// setzt den Cursor an die angegebene Position (y = Zeile, x = Spalte)
extern void LCD_setCursorPos(const unsigned char y, const unsigned char x);

#endif
